@@ -25,7 +25,7 @@
 #include <aws/common/ref_count.h>
 #include <aws/common/string.h>
 
-#if _MSC_VER
+#ifdef _MSC_VER
 #    pragma warning(disable : 4232) /* function pointer to dll symbol */
 #endif
 
