@@ -17,7 +17,7 @@
 
 #include <inttypes.h>
 
-#if _MSC_VER
+#ifdef _MSC_VER
 #    pragma warning(disable : 4204) /* non-constant aggregate initializer */
 #endif
 
@@ -215,6 +215,7 @@ int aws_websocket_client_connect(const struct aws_websocket_client_connection_op
     http_options.on_setup = s_ws_bootstrap_on_http_setup;
     http_options.on_shutdown = s_ws_bootstrap_on_http_shutdown;
     http_options.requested_event_loop = options->requested_event_loop;
+    http_options.host_resolution_config = options->host_resolution_config;
 
     /* Infer port, if not explicitly specified in URI */
     http_options.port = options->port;
