@@ -128,7 +128,7 @@ static void s_testing_channel_shutdown(int error_code, void *user_data) {
         return;
     }
     if (fake_connection->options.on_shutdown) {
-        /* In real world, this is trigger by the bootstrp */
+        /* In real world, this is trigger by the bootstrap */
         fake_connection->options.on_shutdown(
             fake_connection->connection, error_code, fake_connection->options.user_data);
     }
@@ -1399,12 +1399,14 @@ static int s_sm_stream_acquiring_with_body(int num_streams) {
 TEST_CASE(localhost_integ_h2_sm_acquire_stream_stress_with_body) {
     (void)ctx;
     struct aws_byte_cursor uri_cursor = aws_byte_cursor_from_c_str("https://localhost:3443/upload_test");
+    enum aws_log_level log_level = AWS_LOG_LEVEL_DEBUG;
     struct sm_tester_options options = {
         .max_connections = 100,
         .max_concurrent_streams_per_connection = 100,
         .connection_ping_period_ms = 100 * AWS_TIMESTAMP_MILLIS,
         .alloc = allocator,
         .uri_cursor = &uri_cursor,
+        .log_level = &log_level,
     };
     ASSERT_SUCCESS(s_tester_init(&options));
     s_tester.length_sent = 2000;
