@@ -37,7 +37,7 @@
 #include <inttypes.h>
 #include <math.h>
 
-#if _MSC_VER
+#ifdef _MSC_VER
 #    pragma warning(disable : 4232) /* function pointer to dll symbol */
 #endif                              /* _MSC_VER */
 
@@ -66,17 +66,20 @@ const uint32_t g_max_num_connections_per_vip = 10;
 
 /**
  * Default part size is 8 MiB to reach the best performance from the experiments we had.
- * Default max part size is SIZE_MAX on 32bit systems, which is around 4GiB; and 5GiB on a 64bit system.
- *      The server limit is 5GiB, but object size limit is 5TiB for now. We should be good enough for all the cases.
- *      The max number of upload parts is 10000, which limits the object size to 39TiB on 32bit and 49TiB on 64bit.
+ * Default max part size is 5GiB as the server limit. Object size limit is 5TiB for now.
+ *        max number of upload parts is 10000.
  * TODO Provide more information on other values.
  */
 static const size_t s_default_part_size = 8 * 1024 * 1024;
-static const uint64_t s_default_max_part_size = SIZE_MAX < 5368709120ULL ? SIZE_MAX : 5368709120ULL;
+static const uint64_t s_default_max_part_size = 5368709120ULL;
 static const double s_default_throughput_target_gbps = 10.0;
 static const uint32_t s_default_max_retries = 5;
 static size_t s_dns_host_address_ttl_seconds = 5 * 60;
 
+/* Default time until a connection is declared dead, while handling a request but seeing no activity.
+ * 30 seconds mirrors the value currently used by the Java SDK. */
+static const uint32_t s_default_throughput_failure_interval_seconds = 30;
+
 /* Called when ref count is 0. */
 static void s_s3_client_start_destroy(void *user_data);
 
@@ -290,27 +293,29 @@ struct aws_s3_client *aws_s3_client_new(
     client->region = aws_string_new_from_array(allocator, client_config->region.ptr, client_config->region.len);
 
     if (client_config->part_size != 0) {
-        *((size_t *)&client->part_size) = client_config->part_size;
+        *((size_t *)&client->part_size) = (size_t)client_config->part_size;
     } else {
         *((size_t *)&client->part_size) = s_default_part_size;
     }
 
     if (client_config->max_part_size != 0) {
-        *((size_t *)&client->max_part_size) = client_config->max_part_size;
+        *((uint64_t *)&client->max_part_size) = client_config->max_part_size;
     } else {
-        *((size_t *)&client->max_part_size) = (size_t)s_default_max_part_size;
+        *((uint64_t *)&client->max_part_size) = s_default_max_part_size;
+    }
+    if (client->max_part_size > SIZE_MAX) {
+        /* For the 32bit max part size to be SIZE_MAX */
+        *((uint64_t *)&client->max_part_size) = SIZE_MAX;
+    }
+
+    if (client_config->multipart_upload_threshold != 0) {
+        *((uint64_t *)&client->multipart_upload_threshold) = client_config->multipart_upload_threshold;
     }
 
     if (client_config->max_part_size < client_config->part_size) {
-        *((size_t *)&client_config->max_part_size) = client_config->part_size;
+        *((uint64_t *)&client_config->max_part_size) = client_config->part_size;
     }
 
-    if (client_config->proxy_options) {
-        client->proxy_config = aws_http_proxy_config_new_from_proxy_options(allocator, client_config->proxy_options);
-        if (client->proxy_config == NULL) {
-            goto on_error;
-        }
-    }
     client->connect_timeout_ms = client_config->connect_timeout_ms;
     if (client_config->proxy_ev_settings) {
         client->proxy_ev_settings = aws_mem_calloc(allocator, 1, sizeof(struct proxy_env_var_settings));
@@ -331,9 +336,11 @@ struct aws_s3_client *aws_s3_client_new(
     }
 
     if (client_config->monitoring_options) {
-        client->monitoring_options =
-            aws_mem_calloc(allocator, 1, sizeof(struct aws_http_connection_monitoring_options));
-        *client->monitoring_options = *client_config->monitoring_options;
+        client->monitoring_options = *client_config->monitoring_options;
+    } else {
+        client->monitoring_options.minimum_throughput_bytes_per_second = 1;
+        client->monitoring_options.allowable_throughput_failure_interval_seconds =
+            s_default_throughput_failure_interval_seconds;
     }
 
     if (client_config->tls_mode == AWS_MR_TLS_ENABLED) {
@@ -365,6 +372,14 @@ struct aws_s3_client *aws_s3_client_new(
         }
     }
 
+    if (client_config->proxy_options) {
+        client->proxy_config = aws_http_proxy_config_new_from_proxy_options_with_tls_info(
+            allocator, client_config->proxy_options, client_config->tls_mode == AWS_MR_TLS_ENABLED);
+        if (client->proxy_config == NULL) {
+            goto on_error;
+        }
+    }
+
     /* Set up body streaming ELG */
     {
         uint16_t num_event_loops =
@@ -451,6 +466,8 @@ struct aws_s3_client *aws_s3_client_new(
     return client;
 
 on_error:
+    aws_string_destroy(client->region);
+
     if (client->tls_connection_options) {
         aws_tls_connection_options_clean_up(client->tls_connection_options);
         aws_mem_release(client->allocator, client->tls_connection_options);
@@ -465,7 +482,6 @@ on_error:
         client->proxy_ev_settings->tls_options = NULL;
     }
     aws_mem_release(client->allocator, client->proxy_ev_settings);
-    aws_mem_release(client->allocator, client->monitoring_options);
     aws_mem_release(client->allocator, client->tcp_keep_alive_options);
 
     aws_event_loop_group_release(client->client_bootstrap->event_loop_group);
@@ -553,7 +569,6 @@ static void s_s3_client_finish_destroy_default(struct aws_s3_client *client) {
         client->proxy_ev_settings->tls_options = NULL;
     }
     aws_mem_release(client->allocator, client->proxy_ev_settings);
-    aws_mem_release(client->allocator, client->monitoring_options);
     aws_mem_release(client->allocator, client->tcp_keep_alive_options);
 
     aws_mutex_clean_up(&client->synced_data.lock);
@@ -668,13 +683,7 @@ int s_apply_endpoint_override(
     }
 
     struct aws_byte_cursor host_value;
-    if (aws_http_headers_get(message_headers, g_host_header_name, &host_value)) {
-        AWS_LOGF_ERROR(
-            AWS_LS_S3_CLIENT,
-            "id=%p Cannot create meta s3 request; message provided in options does not have a 'Host' header.",
-            (void *)client);
-        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-    }
+    AWS_FATAL_ASSERT(aws_http_headers_get(message_headers, g_host_header_name, &host_value) == AWS_OP_SUCCESS);
 
     if (endpoint_authority != NULL && !aws_byte_cursor_eq(&host_value, endpoint_authority)) {
         AWS_LOGF_ERROR(
@@ -702,8 +711,7 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
     AWS_PRECONDITION(client->vtable->meta_request_factory);
     AWS_PRECONDITION(options);
 
-    if (options->type != AWS_S3_META_REQUEST_TYPE_DEFAULT && options->type != AWS_S3_META_REQUEST_TYPE_GET_OBJECT &&
-        options->type != AWS_S3_META_REQUEST_TYPE_PUT_OBJECT && options->type != AWS_S3_META_REQUEST_TYPE_COPY_OBJECT) {
+    if (options->type >= AWS_S3_META_REQUEST_TYPE_MAX) {
         AWS_LOGF_ERROR(
             AWS_LS_S3_CLIENT,
             "id=%p Cannot create meta s3 request; invalid meta request type specified.",
@@ -783,14 +791,8 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
     }
 
     struct aws_byte_cursor host_header_value;
-    if (aws_http_headers_get(message_headers, g_host_header_name, &host_header_value)) {
-        AWS_LOGF_ERROR(
-            AWS_LS_S3_CLIENT,
-            "id=%p Cannot create meta s3 request; message provided in options does not have a 'Host' header.",
-            (void *)client);
-        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-        return NULL;
-    }
+    /* The Host header must be set from s_apply_endpoint_override, if not errored out */
+    AWS_FATAL_ASSERT(aws_http_headers_get(message_headers, g_host_header_name, &host_header_value) == AWS_OP_SUCCESS);
 
     bool is_https = true;
     uint16_t port = 0;
@@ -868,7 +870,8 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
                 .proxy_ev_settings = client->proxy_ev_settings,
                 .connect_timeout_ms = client->connect_timeout_ms,
                 .tcp_keep_alive_options = client->tcp_keep_alive_options,
-                .monitoring_options = client->monitoring_options};
+                .monitoring_options = &client->monitoring_options,
+            };
 
             endpoint = aws_s3_endpoint_new(client->allocator, &endpoint_options);
 
@@ -954,133 +957,114 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
     }
 
     /* Call the appropriate meta-request new function. */
-    if (options->type == AWS_S3_META_REQUEST_TYPE_GET_OBJECT) {
-
-        /* If the initial request already has partNumber, the request is not splittable(?). Treat it as a Default
-         * request.
-         * TODO: Still need tests to verify that the request of a part is splittable or not */
-        if (aws_http_headers_has(initial_message_headers, aws_byte_cursor_from_c_str("partNumber"))) {
-            return aws_s3_meta_request_default_new(client->allocator, client, content_length, false, options);
-        }
-
-        return aws_s3_meta_request_auto_ranged_get_new(client->allocator, client, client->part_size, options);
-    } else if (options->type == AWS_S3_META_REQUEST_TYPE_PUT_OBJECT) {
-
-        if (!content_length_header_found) {
-            AWS_LOGF_ERROR(
-                AWS_LS_S3_META_REQUEST,
-                "Could not create auto-ranged-put meta request; there is no Content-Length header present.");
-            aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-            return NULL;
-        }
-
-        struct aws_input_stream *input_stream = aws_http_message_get_body_stream(options->message);
-
-        if (input_stream == NULL) {
-            AWS_LOGF_ERROR(
-                AWS_LS_S3_META_REQUEST, "Could not create auto-ranged-put meta request; body stream is NULL.");
-            aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-            return NULL;
-        }
-
-        if (options->resume_token == NULL) {
-
-            size_t client_part_size = client->part_size;
-            size_t client_max_part_size = client->max_part_size;
-
-            if (client_part_size < g_s3_min_upload_part_size) {
-                AWS_LOGF_WARN(
-                    AWS_LS_S3_META_REQUEST,
-                    "Client config part size of %" PRIu64 " is less than the minimum upload part size of %" PRIu64
-                    ". Using to the minimum part-size for upload.",
-                    (uint64_t)client_part_size,
-                    (uint64_t)g_s3_min_upload_part_size);
-
-                client_part_size = g_s3_min_upload_part_size;
+    switch (options->type) {
+        case AWS_S3_META_REQUEST_TYPE_GET_OBJECT: {
+            /* If the initial request already has partNumber, the request is not
+             * splittable(?). Treat it as a Default request.
+             * TODO: Still need tests to verify that the request of a part is
+             * splittable or not */
+            if (aws_http_headers_has(initial_message_headers, aws_byte_cursor_from_c_str("partNumber"))) {
+                return aws_s3_meta_request_default_new(client->allocator, client, content_length, false, options);
             }
 
-            if (client_max_part_size < g_s3_min_upload_part_size) {
-                AWS_LOGF_WARN(
-                    AWS_LS_S3_META_REQUEST,
-                    "Client config max part size of %" PRIu64 " is less than the minimum upload part size of %" PRIu64
-                    ". Clamping to the minimum part-size for upload.",
-                    (uint64_t)client_max_part_size,
-                    (uint64_t)g_s3_min_upload_part_size);
+            return aws_s3_meta_request_auto_ranged_get_new(client->allocator, client, client->part_size, options);
+        }
+        case AWS_S3_META_REQUEST_TYPE_PUT_OBJECT: {
 
-                client_max_part_size = g_s3_min_upload_part_size;
+            if (!content_length_header_found) {
+                AWS_LOGF_ERROR(
+                    AWS_LS_S3_META_REQUEST,
+                    "Could not create auto-ranged-put meta request; there is no Content-Length header present.");
+                aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+                return NULL;
             }
-            if (content_length <= client_part_size) {
-                return aws_s3_meta_request_default_new(
-                    client->allocator,
-                    client,
-                    content_length,
-                    client->compute_content_md5 == AWS_MR_CONTENT_MD5_ENABLED &&
-                        !aws_http_headers_has(initial_message_headers, g_content_md5_header_name),
-                    options);
-            } else {
-                if (aws_s3_message_util_check_checksum_header(options->message)) {
-                    /* The checksum header has been set and the request will be splitted. We fail the request */
-                    AWS_LOGF_ERROR(
+
+            struct aws_input_stream *input_stream = aws_http_message_get_body_stream(options->message);
+
+            if ((input_stream == NULL) && (options->send_filepath.len == 0)) {
+                AWS_LOGF_ERROR(
+                    AWS_LS_S3_META_REQUEST,
+                    "Could not create auto-ranged-put meta request; filepath or body stream must be set.");
+                aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+                return NULL;
+            }
+
+            if (options->resume_token == NULL) {
+
+                size_t client_part_size = client->part_size;
+                uint64_t client_max_part_size = client->max_part_size;
+
+                if (client_part_size < g_s3_min_upload_part_size) {
+                    AWS_LOGF_WARN(
                         AWS_LS_S3_META_REQUEST,
-                        "Could not create auto-ranged-put meta request; checksum headers has been set for "
-                        "auto-ranged-put that will be split. Pre-calculated checksums are only supported for single "
-                        "part upload.");
-                    aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+                        "Client config part size of %" PRIu64 " is less than the minimum upload part size of %" PRIu64
+                        ". Using to the minimum part-size for upload.",
+                        (uint64_t)client_part_size,
+                        (uint64_t)g_s3_min_upload_part_size);
+
+                    client_part_size = g_s3_min_upload_part_size;
+                }
+
+                if (client_max_part_size < (uint64_t)g_s3_min_upload_part_size) {
+                    AWS_LOGF_WARN(
+                        AWS_LS_S3_META_REQUEST,
+                        "Client config max part size of %" PRIu64
+                        " is less than the minimum upload part size of %" PRIu64
+                        ". Clamping to the minimum part-size for upload.",
+                        (uint64_t)client_max_part_size,
+                        (uint64_t)g_s3_min_upload_part_size);
+
+                    client_max_part_size = (uint64_t)g_s3_min_upload_part_size;
+                }
+                uint64_t multipart_upload_threshold =
+                    client->multipart_upload_threshold == 0 ? client_part_size : client->multipart_upload_threshold;
+                if (content_length <= multipart_upload_threshold) {
+                    return aws_s3_meta_request_default_new(
+                        client->allocator,
+                        client,
+                        content_length,
+                        client->compute_content_md5 == AWS_MR_CONTENT_MD5_ENABLED &&
+                            !aws_http_headers_has(initial_message_headers, g_content_md5_header_name),
+                        options);
+                } else {
+                    if (aws_s3_message_util_check_checksum_header(options->message)) {
+                        /* The checksum header has been set and the request will be splitted. We fail the request */
+                        AWS_LOGF_ERROR(
+                            AWS_LS_S3_META_REQUEST,
+                            "Could not create auto-ranged-put meta request; checksum headers has been set for "
+                            "auto-ranged-put that will be split. Pre-calculated checksums are only supported for "
+                            "single "
+                            "part upload.");
+                        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+                        return NULL;
+                    }
+                }
+
+                size_t part_size;
+                uint32_t num_parts;
+                if (aws_s3_calculate_optimal_mpu_part_size_and_num_parts(
+                        content_length, client_part_size, client_max_part_size, &part_size, &num_parts)) {
                     return NULL;
                 }
+
+                return aws_s3_meta_request_auto_ranged_put_new(
+                    client->allocator, client, part_size, content_length, num_parts, options);
+            } else {
+                /* dont pass part size and total num parts. constructor will pick it up from token */
+                return aws_s3_meta_request_auto_ranged_put_new(
+                    client->allocator, client, 0, content_length, 0, options);
             }
-
-            uint64_t part_size_uint64 = content_length / (uint64_t)g_s3_max_num_upload_parts;
-
-            if (part_size_uint64 > SIZE_MAX) {
-                AWS_LOGF_ERROR(
-                    AWS_LS_S3_META_REQUEST,
-                    "Could not create auto-ranged-put meta request; required part size of %" PRIu64
-                    " bytes is too large for platform.",
-                    part_size_uint64);
-
-                aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-                return NULL;
-            }
-
-            size_t part_size = (size_t)part_size_uint64;
-
-            if (part_size > client_max_part_size) {
-                AWS_LOGF_ERROR(
-                    AWS_LS_S3_META_REQUEST,
-                    "Could not create auto-ranged-put meta request; required part size for put request is %" PRIu64
-                    ", but current maximum part size is %" PRIu64,
-                    (uint64_t)part_size,
-                    (uint64_t)client_max_part_size);
-                aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-                return NULL;
-            }
-
-            if (part_size < client_part_size) {
-                part_size = client_part_size;
-            }
-
-            uint32_t num_parts = (uint32_t)(content_length / part_size);
-
-            if ((content_length % part_size) > 0) {
-                ++num_parts;
-            }
-
-            return aws_s3_meta_request_auto_ranged_put_new(
-                client->allocator, client, part_size, content_length, num_parts, options);
-        } else {
-            /* dont pass part size and total num parts. constructor will pick it up from token */
-            return aws_s3_meta_request_auto_ranged_put_new(client->allocator, client, 0, content_length, 0, options);
         }
-    } else if (options->type == AWS_S3_META_REQUEST_TYPE_COPY_OBJECT) {
-        /* TODO: support copy object correctly. */
-        AWS_LOGF_ERROR(AWS_LS_S3_META_REQUEST, "CopyObject is not currently supported");
-        aws_raise_error(AWS_ERROR_UNIMPLEMENTED);
-        return NULL;
-    } else if (options->type == AWS_S3_META_REQUEST_TYPE_DEFAULT) {
-        return aws_s3_meta_request_default_new(client->allocator, client, content_length, false, options);
-    } else {
-        AWS_FATAL_ASSERT(false);
+        case AWS_S3_META_REQUEST_TYPE_COPY_OBJECT: {
+            /* TODO: support copy object correctly. */
+            AWS_LOGF_ERROR(AWS_LS_S3_META_REQUEST, "CopyObject is not currently supported");
+            aws_raise_error(AWS_ERROR_UNIMPLEMENTED);
+            return NULL;
+        }
+        case AWS_S3_META_REQUEST_TYPE_DEFAULT:
+            return aws_s3_meta_request_default_new(client->allocator, client, content_length, false, options);
+        default:
+            AWS_FATAL_ASSERT(false);
     }
 
     return NULL;
@@ -1578,9 +1562,9 @@ static void s_s3_client_create_connection_for_request_default(
     struct aws_http_headers *message_headers = aws_http_message_get_headers(meta_request->initial_request_message);
     AWS_ASSERT(message_headers);
 
-    int get_header_result = aws_http_headers_get(message_headers, g_host_header_name, &host_header_value);
-    AWS_ASSERT(get_header_result == AWS_OP_SUCCESS);
-    (void)get_header_result;
+    int result = aws_http_headers_get(message_headers, g_host_header_name, &host_header_value);
+    AWS_ASSERT(result == AWS_OP_SUCCESS);
+    (void)result;
 
     if (aws_retry_strategy_acquire_retry_token(
             client->retry_strategy, &host_header_value, s_s3_client_acquired_retry_token, connection, 0)) {
@@ -1732,6 +1716,9 @@ void aws_s3_client_notify_connection_finished(
 
     struct aws_s3_endpoint *endpoint = meta_request->endpoint;
     AWS_PRECONDITION(endpoint);
+    if (request->send_data.metrics) {
+        request->send_data.metrics->crt_info_metrics.error_code = error_code;
+    }
 
     /* If we're trying to setup a retry... */
     if (finish_code == AWS_S3_CONNECTION_FINISH_CODE_RETRY) {
@@ -1739,7 +1726,7 @@ void aws_s3_client_notify_connection_finished(
         if (connection->retry_token == NULL) {
             AWS_LOGF_ERROR(
                 AWS_LS_S3_CLIENT,
-                "id=%p Client could not schedule retry of request %p for meta request %p",
+                "id=%p Client could not schedule retry of request %p for meta request %p, as retry token is NULL.",
                 (void *)client,
                 (void *)request,
                 (void *)meta_request);
@@ -1976,10 +1963,14 @@ struct aws_s3_meta_request_resume_token *aws_s3_meta_request_resume_token_new_up
     const struct aws_s3_upload_resume_token_options *options) {
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(options);
+    if (options->part_size > SIZE_MAX) {
+        aws_raise_error(AWS_ERROR_OVERFLOW_DETECTED);
+        return NULL;
+    }
 
     struct aws_s3_meta_request_resume_token *token = aws_s3_meta_request_resume_token_new(allocator);
     token->multipart_upload_id = aws_string_new_from_cursor(allocator, &options->upload_id);
-    token->part_size = options->part_size;
+    token->part_size = (size_t)options->part_size;
     token->total_num_parts = options->total_num_parts;
     token->num_parts_completed = options->num_parts_completed;
     token->type = AWS_S3_META_REQUEST_TYPE_PUT_OBJECT;
@@ -2008,9 +1999,9 @@ enum aws_s3_meta_request_type aws_s3_meta_request_resume_token_type(
     return resume_token->type;
 }
 
-size_t aws_s3_meta_request_resume_token_part_size(struct aws_s3_meta_request_resume_token *resume_token) {
+uint64_t aws_s3_meta_request_resume_token_part_size(struct aws_s3_meta_request_resume_token *resume_token) {
     AWS_FATAL_PRECONDITION(resume_token);
-    return resume_token->part_size;
+    return (uint64_t)resume_token->part_size;
 }
 
 size_t aws_s3_meta_request_resume_token_total_num_parts(struct aws_s3_meta_request_resume_token *resume_token) {
