@@ -7,6 +7,7 @@
 #include "aws/s3/private/s3_checksums.h"
 #include "aws/s3/private/s3_client_impl.h"
 #include "aws/s3/private/s3_meta_request_impl.h"
+#include "aws/s3/private/s3_request_messages.h"
 #include "aws/s3/private/s3_util.h"
 #include <aws/auth/signable.h>
 #include <aws/auth/signing.h>
@@ -17,6 +18,7 @@
 #include <aws/common/system_info.h>
 #include <aws/io/event_loop.h>
 #include <aws/io/retry_strategy.h>
+#include <aws/io/socket.h>
 #include <aws/io/stream.h>
 #include <inttypes.h>
 
@@ -53,6 +55,11 @@ static int s_s3_meta_request_incoming_headers(
     size_t headers_count,
     void *user_data);
 
+static void s_s3_meta_request_stream_metrics(
+    struct aws_http_stream *stream,
+    const struct aws_http_stream_metrics *metrics,
+    void *user_data);
+
 static void s_s3_meta_request_stream_complete(struct aws_http_stream *stream, int error_code, void *user_data);
 
 static void s_s3_meta_request_send_request_finish(
@@ -178,8 +185,6 @@ int aws_s3_meta_request_init_base(
     AWS_PRECONDITION(meta_request);
 
     AWS_ZERO_STRUCT(*meta_request);
-    meta_request->impl = impl;
-    meta_request->vtable = vtable;
 
     AWS_ASSERT(vtable->update);
     AWS_ASSERT(vtable->prepare_request);
@@ -191,10 +196,32 @@ int aws_s3_meta_request_init_base(
 
     meta_request->allocator = allocator;
     meta_request->type = options->type;
-
     /* Set up reference count. */
     aws_ref_count_init(&meta_request->ref_count, meta_request, s_s3_meta_request_destroy);
 
+    if (part_size == SIZE_MAX) {
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        goto error;
+    }
+
+    if (aws_mutex_init(&meta_request->synced_data.lock)) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST, "id=%p Could not initialize mutex for meta request", (void *)meta_request);
+        goto error;
+    }
+
+    if (aws_priority_queue_init_dynamic(
+            &meta_request->synced_data.pending_body_streaming_requests,
+            meta_request->allocator,
+            s_default_body_streaming_priority_queue_size,
+            sizeof(struct aws_s3_request *),
+            s_s3_request_priority_queue_pred)) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST, "id=%p Could not initialize priority queue for meta request", (void *)meta_request);
+        /* Priority queue */
+        goto error;
+    }
+
     *((size_t *)&meta_request->part_size) = part_size;
     *((bool *)&meta_request->should_compute_content_md5) = should_compute_content_md5;
     checksum_config_init(&meta_request->checksum_config, options->checksum_config);
@@ -202,23 +229,19 @@ int aws_s3_meta_request_init_base(
         meta_request->cached_signing_config = aws_cached_signing_config_new(allocator, options->signing_config);
     }
 
-    /* Keep a reference to the original message structure passed in. */
-    meta_request->initial_request_message = options->message;
-    aws_http_message_acquire(options->message);
-
-    if (aws_mutex_init(&meta_request->synced_data.lock)) {
-        AWS_LOGF_ERROR(
-            AWS_LS_S3_META_REQUEST, "id=%p Could not initialize mutex for meta request", (void *)meta_request);
-        return AWS_OP_ERR;
+    /* Set initial_meta_request */
+    if (options->send_filepath.len > 0) {
+        /* Create copy of original message, but with body-stream that reads directly from file */
+        meta_request->initial_request_message = aws_s3_message_util_copy_http_message_filepath_body_all_headers(
+            allocator, options->message, options->send_filepath);
+        if (meta_request->initial_request_message == NULL) {
+            goto error;
+        }
+    } else {
+        /* Keep a reference to the original message structure passed in. */
+        meta_request->initial_request_message = aws_http_message_acquire(options->message);
     }
 
-    aws_priority_queue_init_dynamic(
-        &meta_request->synced_data.pending_body_streaming_requests,
-        meta_request->allocator,
-        s_default_body_streaming_priority_queue_size,
-        sizeof(struct aws_s3_request *),
-        s_s3_request_priority_queue_pred);
-
     /* Client is currently optional to allow spinning up a meta_request without a client in a test. */
     if (client != NULL) {
         aws_s3_client_acquire(client);
@@ -233,6 +256,7 @@ int aws_s3_meta_request_init_base(
     meta_request->user_data = options->user_data;
     meta_request->shutdown_callback = options->shutdown_callback;
     meta_request->progress_callback = options->progress_callback;
+    meta_request->telemetry_callback = options->telemetry_callback;
 
     if (meta_request->checksum_config.validate_response_checksum) {
         /* TODO: the validate for auto range get should happen for each response received. */
@@ -249,7 +273,14 @@ int aws_s3_meta_request_init_base(
         meta_request->finish_callback = options->finish_callback;
     }
 
+    /* Nothing can fail after here. Leave the impl not affected by failure of initializing base. */
+    meta_request->impl = impl;
+    meta_request->vtable = vtable;
+
     return AWS_OP_SUCCESS;
+error:
+    s_s3_meta_request_destroy((void *)meta_request);
+    return AWS_OP_ERR;
 }
 
 void aws_s3_meta_request_increment_read_window(struct aws_s3_meta_request *meta_request, uint64_t bytes) {
@@ -409,14 +440,14 @@ static void s_s3_meta_request_destroy(void *user_data) {
     aws_s3_endpoint_release(meta_request->endpoint);
     meta_request->client = aws_s3_client_release(meta_request->client);
 
-    AWS_ASSERT(aws_priority_queue_size(&meta_request->synced_data.pending_body_streaming_requests) == 0);
     aws_priority_queue_clean_up(&meta_request->synced_data.pending_body_streaming_requests);
     aws_s3_meta_request_result_clean_up(meta_request, &meta_request->synced_data.finish_result);
 
-    AWS_LOGF_TRACE(
-        AWS_LS_S3_META_REQUEST, "id=%p Calling virtual meta request destroy function.", (void *)meta_request);
-
-    meta_request->vtable->destroy(meta_request);
+    if (meta_request->vtable != NULL) {
+        AWS_LOGF_TRACE(
+            AWS_LS_S3_META_REQUEST, "id=%p Calling virtual meta request destroy function.", (void *)meta_request);
+        meta_request->vtable->destroy(meta_request);
+    }
     meta_request = NULL;
 
     if (shutdown_callback != NULL) {
@@ -791,6 +822,9 @@ void aws_s3_meta_request_send_request(struct aws_s3_meta_request *meta_request,
     options.on_response_headers = s_s3_meta_request_incoming_headers;
     options.on_response_header_block_done = NULL;
     options.on_response_body = s_s3_meta_request_incoming_body;
+    if (request->send_data.metrics) {
+        options.on_metrics = s_s3_meta_request_stream_metrics;
+    }
     options.on_complete = s_s3_meta_request_stream_complete;
 
     struct aws_http_stream *stream = aws_http_connection_make_request(connection->http_connection, &options);
@@ -950,6 +984,25 @@ static int s_s3_meta_request_incoming_headers(
             (void *)meta_request,
             (void *)request);
     }
+    if (request->send_data.metrics) {
+        /* Record the headers to the metrics */
+        struct aws_s3_request_metrics *s3_metrics = request->send_data.metrics;
+        if (s3_metrics->req_resp_info_metrics.response_headers == NULL) {
+            s3_metrics->req_resp_info_metrics.response_headers = aws_http_headers_new(meta_request->allocator);
+        }
+
+        for (size_t i = 0; i < headers_count; ++i) {
+            const struct aws_byte_cursor *name = &headers[i].name;
+            const struct aws_byte_cursor *value = &headers[i].value;
+            if (aws_byte_cursor_eq(name, &g_request_id_header_name)) {
+                s3_metrics->req_resp_info_metrics.request_id =
+                    aws_string_new_from_cursor(connection->request->allocator, value);
+            }
+
+            aws_http_headers_add(s3_metrics->req_resp_info_metrics.response_headers, *name, *value);
+        }
+        s3_metrics->req_resp_info_metrics.response_status = request->send_data.response_status;
+    }
 
     bool successful_response =
         s_s3_meta_request_error_code_from_response_status(request->send_data.response_status) == AWS_ERROR_SUCCESS;
@@ -1036,6 +1089,38 @@ static int s_s3_meta_request_incoming_body(
     return AWS_OP_SUCCESS;
 }
 
+static void s_s3_meta_request_stream_metrics(
+    struct aws_http_stream *stream,
+    const struct aws_http_stream_metrics *http_metrics,
+    void *user_data) {
+    (void)stream;
+    struct aws_s3_connection *connection = user_data;
+    AWS_PRECONDITION(connection);
+
+    struct aws_s3_request *request = connection->request;
+    AWS_PRECONDITION(request);
+    AWS_ASSERT(request->send_data.metrics);
+    struct aws_s3_request_metrics *s3_metrics = request->send_data.metrics;
+    /* Copy over the time metrics from aws_http_stream_metrics to aws_s3_request_metrics */
+    s3_metrics->time_metrics.send_start_timestamp_ns = http_metrics->send_start_timestamp_ns;
+    s3_metrics->time_metrics.send_end_timestamp_ns = http_metrics->send_end_timestamp_ns;
+    s3_metrics->time_metrics.sending_duration_ns = http_metrics->sending_duration_ns;
+    s3_metrics->time_metrics.receive_start_timestamp_ns = http_metrics->receive_start_timestamp_ns;
+    s3_metrics->time_metrics.receive_end_timestamp_ns = http_metrics->receive_end_timestamp_ns;
+    s3_metrics->time_metrics.receiving_duration_ns = http_metrics->receiving_duration_ns;
+
+    s3_metrics->crt_info_metrics.stream_id = http_metrics->stream_id;
+
+    /* Also related metrics from the request/response. */
+    s3_metrics->crt_info_metrics.connection_id = (void *)connection->http_connection;
+    const struct aws_socket_endpoint *endpoint = aws_http_connection_get_remote_endpoint(connection->http_connection);
+    request->send_data.metrics->crt_info_metrics.ip_address =
+        aws_string_new_from_c_str(request->allocator, endpoint->address);
+    AWS_ASSERT(request->send_data.metrics->crt_info_metrics.ip_address != NULL);
+
+    s3_metrics->crt_info_metrics.thread_id = aws_thread_current_thread_id();
+}
+
 /* Finish up the processing of the request work. */
 static void s_s3_meta_request_stream_complete(struct aws_http_stream *stream, int error_code, void *user_data) {
 
