@@ -59,6 +59,7 @@ enum aws_s3_client_tls_usage {
 enum aws_s3_tester_validate_type {
     AWS_S3_TESTER_VALIDATE_TYPE_EXPECT_SUCCESS,
     AWS_S3_TESTER_VALIDATE_TYPE_EXPECT_FAILURE,
+    AWS_S3_TESTER_VALIDATE_TYPE_NO_VALIDATE,
 };
 
 enum aws_s3_tester_default_type_mode {
@@ -92,6 +93,9 @@ struct aws_s3_tester {
     struct aws_array_list meta_request_vtable_patches;
     void *user_data;
 
+    struct aws_string *bucket_name;
+    struct aws_string *public_bucket_name;
+
     struct {
         struct aws_mutex lock;
 
@@ -117,9 +121,10 @@ struct aws_s3_tester {
 
 struct aws_s3_tester_client_options {
     enum aws_s3_client_tls_usage tls_usage;
-    size_t part_size;
+    uint64_t part_size;
     size_t max_part_size;
     uint32_t setup_region : 1;
+    uint32_t use_proxy : 1;
 };
 
 /* should really break this up to a client setup, and a meta_request sending */
@@ -177,8 +182,12 @@ struct aws_s3_tester_meta_request_options {
         bool invalid_input_stream;
         bool valid_md5;
         bool invalid_md5;
+        /* write file to desk, and then send via aws_s3_meta_request_options.send_filepath */
+        bool file_on_disk;
+        struct aws_s3_meta_request_resume_token *resume_token;
         /* manually overwrite the content length for some invalid input stream */
         size_t content_length;
+        struct aws_byte_cursor content_encoding;
     } put_options;
 
     enum aws_s3_tester_sse_type sse_type;
@@ -212,6 +221,15 @@ struct aws_s3_meta_request_test_results {
     int finished_response_status;
     int finished_error_code;
     enum aws_s3_checksum_algorithm algorithm;
+
+    /* accumulator of amount of bytes uploaded */
+    struct aws_atomic_var total_bytes_uploaded;
+
+    /* Protected the tester->synced_data.lock */
+    struct {
+        /* The array_list of `struct aws_s3_request_metrics *` */
+        struct aws_array_list metrics;
+    } synced_data;
 };
 
 struct aws_s3_client_config;
@@ -292,12 +310,20 @@ struct aws_http_message *aws_s3_test_get_object_request_new(
 
 struct aws_http_message *aws_s3_test_put_object_request_new(
     struct aws_allocator *allocator,
-    struct aws_byte_cursor host,
+    struct aws_byte_cursor *host,
     struct aws_byte_cursor content_type,
     struct aws_byte_cursor key,
     struct aws_input_stream *body_stream,
     uint32_t flags);
 
+struct aws_http_message *aws_s3_test_put_object_request_new_without_body(
+    struct aws_allocator *allocator,
+    struct aws_byte_cursor *host,
+    struct aws_byte_cursor content_type,
+    struct aws_byte_cursor key,
+    uint64_t content_length,
+    uint32_t flags);
+
 int aws_s3_tester_client_new(
     struct aws_s3_tester *tester,
     struct aws_s3_tester_client_options *options,
@@ -399,14 +425,14 @@ int aws_s3_tester_upload_file_path_init(
     struct aws_byte_buf *out_path_buffer,
     struct aws_byte_cursor file_path);
 
+int aws_s3_tester_get_content_length(const struct aws_http_headers *headers, uint64_t *out_content_length);
+
 extern struct aws_s3_client_vtable g_aws_s3_client_mock_vtable;
 
 extern const struct aws_byte_cursor g_mock_server_uri;
 
 extern const struct aws_byte_cursor g_test_body_content_type;
 extern const struct aws_byte_cursor g_test_s3_region;
-extern const struct aws_byte_cursor g_test_bucket_name;
-extern const struct aws_byte_cursor g_test_public_bucket_name;
 
 extern const struct aws_byte_cursor g_pre_existing_object_1MB;
 extern const struct aws_byte_cursor g_pre_existing_object_10MB;
@@ -416,4 +442,11 @@ extern const struct aws_byte_cursor g_pre_existing_empty_object;
 
 extern const struct aws_byte_cursor g_put_object_prefix;
 
+/* If `$CRT_S3_TEST_BUCKET_NAME` environment variable is set, use that; otherwise, use aws-c-s3-test-bucket */
+extern struct aws_byte_cursor g_test_bucket_name;
+/* If `$CRT_S3_TEST_BUCKET_NAME` envrionment variable is set, use `$CRT_S3_TEST_BUCKET_NAME-public`; otherwise, use
+ * aws-c-s3-test-bucket-public
+ */
+extern struct aws_byte_cursor g_test_public_bucket_name;
+
 #endif /* AWS_S3_TESTER_H */
