@@ -41,18 +41,24 @@ fi
 make clean;
 
 echo "Using $JOBS jobs for make..";
+echo "running with libcrypto: ${S2N_LIBCRYPTO}, gcc_version: ${GCC_VERSION}"
 
 if [[ "$OS_NAME" == "linux" && "$TESTS" == "valgrind" ]]; then
     # For linux make a build with debug symbols and run valgrind
     # We have to output something every 9 minutes, as some test may run longer than 10 minutes
     # and will not produce any output
     while sleep 9m; do echo "=====[ $SECONDS seconds still running ]====="; done &
-    S2N_DEBUG=true make -j $JOBS valgrind
-    kill %1
-fi
 
-if [[ "$OS_NAME" == "linux" && ( "$TESTS" == "integrationv2") ]]; then
-    make -j $JOBS
+    if [[ "$S2N_LIBCRYPTO" == "openssl-1.1.1" || "$S2N_LIBCRYPTO" == "awslc" ]]; then
+        # https://github.com/aws/s2n-tls/issues/3758
+        # Run valgrind in pedantic mode (--errors-for-leak-kinds=all)
+        echo "running task pedantic_valgrind"
+        S2N_DEBUG=true make -j $JOBS pedantic_valgrind
+    else
+        S2N_DEBUG=true make -j $JOBS valgrind
+    fi
+
+    kill %1
 fi
 
 CMAKE_PQ_OPTION="S2N_NO_PQ=False"
@@ -85,8 +91,24 @@ setup_apache_server() {
 run_integration_v2_tests() {
     setup_apache_server
     "$CB_BIN_DIR/install_s2n_head.sh" "$(mktemp -d)"
-    make clean
-    make integrationv2
+    cmake . -Bbuild \
+            -DCMAKE_PREFIX_PATH=$LIBCRYPTO_ROOT \
+            -D${CMAKE_PQ_OPTION} \
+            -DS2N_BLOCK_NONPORTABLE_OPTIMIZATIONS=True \
+            -DBUILD_SHARED_LIBS=on \
+            -DS2N_INTEG_TESTS=on \
+            -DPython3_EXECUTABLE=$(which python3)
+    cmake --build ./build --clean-first -- -j $(nproc)
+    test_linked_libcrypto ./build/bin/s2nc
+    test_linked_libcrypto ./build/bin/s2nd
+    cp -f ./build/bin/s2nc "$BASE_S2N_DIR"/bin/s2nc
+    cp -f ./build/bin/s2nd "$BASE_S2N_DIR"/bin/s2nd
+    cd ./build/
+    for test_name in $TOX_TEST_NAME; do
+      test="${test_name//test_/}"
+      echo "Running... ctest --no-tests=error --output-on-failure --verbose -R ^integrationv2_${test}$"
+      ctest --no-tests=error --output-on-failure --verbose -R ^integrationv2_${test}$
+    done
 }
 
 run_unit_tests() {
@@ -94,10 +116,11 @@ run_unit_tests() {
             -DCMAKE_PREFIX_PATH=$LIBCRYPTO_ROOT \
             -D${CMAKE_PQ_OPTION} \
             -DS2N_BLOCK_NONPORTABLE_OPTIMIZATIONS=True \
-            -DBUILD_SHARED_LIBS=on
+            -DBUILD_SHARED_LIBS=on \
+            -DEXPERIMENTAL_TREAT_WARNINGS_AS_ERRORS=on
     cmake --build ./build -- -j $(nproc)
     test_linked_libcrypto ./build/bin/s2nc
-    make -C build test ARGS=-j$(nproc);
+    cmake --build build/ --target test -- ARGS="-L unit --output-on-failure -j $(nproc)"
 }
 
 # Run Multiple tests on one flag.
