@@ -146,6 +146,8 @@ class Certificates(object):
     RSA_PSS_2048_SHA256 = Cert(
         "RSA_PSS_2048_SHA256", "localhost_rsa_pss_2048_sha256")
 
+    RSA_2048_PKCS1 = Cert("RSA_2048_PKCS1", "rsa_2048_pkcs1")
+
     OCSP = Cert("OCSP_RSA", "ocsp/server")
     OCSP_ECDSA = Cert("OCSP_ECDSA_256", "ocsp/server_ecdsa")
 
@@ -305,6 +307,11 @@ class Ciphers(object):
         "PQ-SIKE-TEST-TLS-1-0-2020-02", Protocols.TLS10, False, False, s2n=True, pq=True)
     PQ_TLS_1_0_2020_12 = Cipher(
         "PQ-TLS-1-0-2020-12", Protocols.TLS10, False, False, s2n=True, pq=True)
+    PQ_TLS_1_0_2023_01 = Cipher(
+        "PQ-TLS-1-0-2023-01-24", Protocols.TLS10, False, False, s2n=True, pq=True)
+
+    SECURITY_POLICY_20210816 = Cipher(
+        "20210816", Protocols.TLS12, False, False, s2n=True, pq=False)
 
     @staticmethod
     def from_iana(iana_name):
@@ -383,6 +390,9 @@ class Signatures(object):
     RSA_SHA256 = Signature('RSA+SHA256', max_protocol=Protocols.TLS12)
     RSA_SHA384 = Signature('RSA+SHA384', max_protocol=Protocols.TLS12)
     RSA_SHA512 = Signature('RSA+SHA512', max_protocol=Protocols.TLS12)
+    RSA_MD5_SHA1 = Signature('RSA+MD5_SHA1', max_protocol=Protocols.TLS11)
+    ECDSA_SHA224 = Signature('ECDSA+SHA224', max_protocol=Protocols.TLS12)
+    ECDSA_SHA1 = Signature('ECDSA+SHA1', max_protocol=Protocols.TLS12)
 
     RSA_PSS_RSAE_SHA256 = Signature(
         'RSA-PSS+SHA256',
