@@ -100,6 +100,15 @@ EXPECTED_RESULTS = {
     (Ciphers.PQ_TLS_1_0_2020_12, Ciphers.PQ_TLS_1_0_2020_12):
         {"cipher": "TLS_AES_256_GCM_SHA384",
             "kem": "NONE", "kem_group": "_kyber-512-r3"},
+    (Ciphers.PQ_TLS_1_0_2020_12, Ciphers.PQ_TLS_1_0_2023_01):
+        {"cipher": "TLS_AES_256_GCM_SHA384",
+            "kem": "NONE", "kem_group": "_kyber-512-r3"},
+    (Ciphers.PQ_TLS_1_0_2023_01, Ciphers.PQ_TLS_1_0_2023_01):
+        {"cipher": "TLS_AES_256_GCM_SHA384",
+            "kem": "NONE", "kem_group": "_kyber-512-r3"},
+    (Ciphers.PQ_TLS_1_0_2023_01, Ciphers.PQ_TLS_1_0_2020_12):
+        {"cipher": "TLS_AES_256_GCM_SHA384",
+            "kem": "NONE", "kem_group": "_kyber-512-r3"},
     (Ciphers.PQ_TLS_1_0_2020_12, Ciphers.KMS_PQ_TLS_1_0_2020_07):
         {"cipher": "ECDHE-KYBER-RSA-AES256-GCM-SHA384",
             "kem": "kyber512r3", "kem_group": "NONE"},
@@ -112,6 +121,9 @@ EXPECTED_RESULTS = {
     (KemGroups.P256_KYBER512R3, Ciphers.PQ_TLS_1_0_2020_12):
         {"cipher": "AES256_GCM_SHA384", "kem": "NONE",
             "kem_group": "secp256r1_kyber-512-r3"},
+    (KemGroups.P256_KYBER512R3, Ciphers.PQ_TLS_1_0_2023_01):
+        {"cipher": "AES256_GCM_SHA384", "kem": "NONE",
+            "kem_group": "secp256r1_kyber-512-r3"},
 }
 
 """
