@@ -113,33 +113,6 @@ int main(int argc, char **argv)
     EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&rsa_chain_and_key,
             S2N_DEFAULT_TEST_CERT_CHAIN, S2N_DEFAULT_TEST_PRIVATE_KEY));
 
-    /* Test s2n_connection does not grow too much.
-     * s2n_connection is a very large structure. We should be working to reduce its
-     * size, not increasing it.
-     * This test documents changes to its size for reviewers so that we can
-     * make very deliberate choices about increasing memory usage.
-     *
-     * We can't easily enforce an exact size for s2n_connection because it varies
-     * based on some settings (like how many KEM groups are supported).
-     */
-    {
-        /* Carefully consider any increases to this number. */
-        const uint16_t max_connection_size = 4250;
-        const uint16_t min_connection_size = max_connection_size * 0.9;
-
-        size_t connection_size = sizeof(struct s2n_connection);
-
-        if (connection_size > max_connection_size || connection_size < min_connection_size) {
-            const char message[] = "s2n_connection size (%zu) no longer in (%i, %i). "
-                                   "Please verify that this change was intentional and then update this test.";
-            char message_buffer[sizeof(message) + 100] = { 0 };
-            int r = snprintf(message_buffer, sizeof(message_buffer), message,
-                    connection_size, min_connection_size, max_connection_size);
-            EXPECT_TRUE(r < sizeof(message_buffer));
-            FAIL_MSG(message_buffer);
-        }
-    };
-
     /* s2n_get_server_name */
     {
         const char *test_server_name = "A server name";
@@ -808,6 +781,107 @@ int main(int argc, char **argv)
         };
     };
 
+    /* Test: s2n_connection_check_io_status */
+    {
+        /* Safety */
+        {
+            DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(conn);
+
+            EXPECT_FALSE(s2n_connection_check_io_status(NULL, S2N_IO_WRITABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(NULL, S2N_IO_READABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(NULL, S2N_IO_FULL_DUPLEX));
+            EXPECT_FALSE(s2n_connection_check_io_status(NULL, S2N_IO_CLOSED));
+            EXPECT_FALSE(s2n_connection_check_io_status(NULL, 10));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, 10));
+
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_WRITABLE));
+            conn->write_closed = 10;
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_WRITABLE));
+
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_READABLE));
+            conn->read_closed = 10;
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_READABLE));
+        }
+
+        /* TLS1.2 */
+        {
+            DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(conn);
+            conn->actual_protocol_version = S2N_TLS12;
+
+            /* Full duplex by default */
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_WRITABLE));
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_READABLE));
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_FULL_DUPLEX));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_CLOSED));
+
+            /* Close write */
+            conn->write_closed = 1;
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_WRITABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_READABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_FULL_DUPLEX));
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_CLOSED));
+            conn->write_closed = 0;
+
+            /* Close read */
+            conn->read_closed = 1;
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_WRITABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_READABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_FULL_DUPLEX));
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_CLOSED));
+            conn->read_closed = 0;
+
+            /* Close both */
+            conn->read_closed = 1;
+            conn->write_closed = 1;
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_WRITABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_READABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_FULL_DUPLEX));
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_CLOSED));
+        };
+
+        /* TLS1.3 */
+        {
+            DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(conn);
+            conn->actual_protocol_version = S2N_TLS13;
+
+            /* Full duplex by default */
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_WRITABLE));
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_READABLE));
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_FULL_DUPLEX));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_CLOSED));
+
+            /* Close write */
+            conn->write_closed = 1;
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_WRITABLE));
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_READABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_FULL_DUPLEX));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_CLOSED));
+            conn->write_closed = 0;
+
+            /* Close read */
+            conn->read_closed = 1;
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_WRITABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_READABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_FULL_DUPLEX));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_CLOSED));
+            conn->read_closed = 0;
+
+            /* Close both */
+            conn->read_closed = 1;
+            conn->write_closed = 1;
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_WRITABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_READABLE));
+            EXPECT_FALSE(s2n_connection_check_io_status(conn, S2N_IO_FULL_DUPLEX));
+            EXPECT_TRUE(s2n_connection_check_io_status(conn, S2N_IO_CLOSED));
+        };
+    };
+
     EXPECT_SUCCESS(s2n_cert_chain_and_key_free(ecdsa_chain_and_key));
     EXPECT_SUCCESS(s2n_cert_chain_and_key_free(rsa_chain_and_key));
     END_TEST();
