@@ -31,6 +31,7 @@ struct s2n_stuffer input_stuffer, output_stuffer;
 static S2N_RESULT s2n_setup_conn(struct s2n_connection *conn)
 {
     conn->actual_protocol_version = S2N_TLS13;
+    EXPECT_OK(s2n_conn_choose_state_machine(conn, S2N_TLS13));
 
     RESULT_GUARD_POSIX(s2n_stuffer_wipe(&input_stuffer));
     RESULT_GUARD_POSIX(s2n_stuffer_wipe(&output_stuffer));
@@ -79,7 +80,7 @@ static S2N_RESULT s2n_write_test_message(struct s2n_blob *out, message_type_t me
 {
     RESULT_GUARD_POSIX(s2n_alloc(out, TEST_DATA_SIZE + TLS_HANDSHAKE_HEADER_LENGTH));
 
-    struct s2n_stuffer stuffer;
+    struct s2n_stuffer stuffer = { 0 };
     RESULT_GUARD_POSIX(s2n_stuffer_init(&stuffer, out));
 
     RESULT_GUARD_POSIX(s2n_stuffer_write_uint8(&stuffer, message_type));
@@ -114,13 +115,7 @@ int main(int argc, char **argv)
     /* Test: s2n_quic_write_handshake_message */
     {
         /* Safety checks */
-        {
-            struct s2n_connection conn = { 0 };
-            struct s2n_blob blob = { 0 };
-
-            EXPECT_ERROR(s2n_quic_write_handshake_message(NULL, &blob));
-            EXPECT_ERROR(s2n_quic_write_handshake_message(&conn, NULL));
-        };
+        EXPECT_ERROR(s2n_quic_write_handshake_message(NULL));
 
         /* Writes handshake message */
         {
@@ -128,10 +123,9 @@ int main(int argc, char **argv)
             EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
 
             uint8_t message_data[] = "The client says hello";
-            struct s2n_blob in;
-            EXPECT_SUCCESS(s2n_blob_init(&in, message_data, sizeof(message_data)));
+            EXPECT_SUCCESS(s2n_stuffer_write_bytes(&conn->handshake.io, message_data, sizeof(message_data)));
 
-            EXPECT_OK(s2n_quic_write_handshake_message(conn, &in));
+            EXPECT_OK(s2n_quic_write_handshake_message(conn));
             EXPECT_EQUAL(s2n_stuffer_data_available(&conn->out), sizeof(message_data));
             EXPECT_BYTEARRAY_EQUAL(s2n_stuffer_raw_read(&conn->out, sizeof(message_data)),
                     message_data, sizeof(message_data));
@@ -156,7 +150,7 @@ int main(int argc, char **argv)
             struct s2n_connection *conn;
             EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
 
-            struct s2n_stuffer stuffer;
+            struct s2n_stuffer stuffer = { 0 };
             EXPECT_SUCCESS(s2n_stuffer_growable_alloc(&stuffer, 0));
             EXPECT_SUCCESS(s2n_connection_set_io_stuffers(&stuffer, &stuffer, conn));
 
@@ -183,7 +177,7 @@ int main(int argc, char **argv)
             struct s2n_connection *conn;
             EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
 
-            struct s2n_stuffer stuffer;
+            struct s2n_stuffer stuffer = { 0 };
             EXPECT_SUCCESS(s2n_stuffer_growable_alloc(&stuffer, 0));
             EXPECT_SUCCESS(s2n_connection_set_io_stuffers(&stuffer, &stuffer, conn));
 
@@ -202,7 +196,7 @@ int main(int argc, char **argv)
             struct s2n_connection *conn;
             EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
 
-            struct s2n_stuffer stuffer;
+            struct s2n_stuffer stuffer = { 0 };
             EXPECT_SUCCESS(s2n_stuffer_growable_alloc(&stuffer, 0));
             EXPECT_SUCCESS(s2n_connection_set_io_stuffers(&stuffer, &stuffer, conn));
 
@@ -223,7 +217,7 @@ int main(int argc, char **argv)
             struct s2n_connection *conn;
             EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
 
-            struct s2n_stuffer stuffer;
+            struct s2n_stuffer stuffer = { 0 };
             EXPECT_SUCCESS(s2n_stuffer_growable_alloc(&stuffer, 0));
             EXPECT_SUCCESS(s2n_connection_set_io_stuffers(&stuffer, &stuffer, conn));
 
@@ -400,7 +394,7 @@ int main(int argc, char **argv)
             EXPECT_SUCCESS(s2n_connection_set_config(conn, config));
 
             /* Sabotage the output stuffer to block writing */
-            struct s2n_stuffer bad_stuffer;
+            struct s2n_stuffer bad_stuffer = { 0 };
             EXPECT_SUCCESS(s2n_connection_set_io_stuffers(&input_stuffer, &bad_stuffer, conn));
 
             EXPECT_FAILURE_WITH_ERRNO(s2n_negotiate(conn, &blocked_status), S2N_ERR_IO_BLOCKED);
