@@ -101,6 +101,7 @@ int main(int argc, char **argv)
 
         /* A HelloRetryRequest only requires a Selected Group, not a key share */
         EXPECT_SUCCESS(s2n_connection_set_all_protocol_versions(conn, S2N_TLS13));
+        EXPECT_OK(s2n_conn_choose_state_machine(conn, S2N_TLS13));
         EXPECT_SUCCESS(s2n_set_connection_hello_retry_flags(conn));
         conn->kex_params.server_ecc_evp_params.negotiated_curve = s2n_all_supported_curves_list[0];
         EXPECT_EQUAL(6, s2n_extensions_server_key_share_send_size(conn));
@@ -146,6 +147,7 @@ int main(int argc, char **argv)
         struct s2n_connection *conn = NULL;
         EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_SERVER));
         conn->actual_protocol_version = S2N_TLS13;
+        EXPECT_OK(s2n_conn_choose_state_machine(conn, S2N_TLS13));
         conn->handshake.handshake_type = HELLO_RETRY_REQUEST;
         conn->handshake.message_number = HELLO_RETRY_MSG_NO;
 
@@ -245,7 +247,7 @@ int main(int argc, char **argv)
             };
 
             for (int i = 0; i < 3; i++) {
-                struct s2n_stuffer extension_stuffer;
+                struct s2n_stuffer extension_stuffer = { 0 };
                 struct s2n_connection *client_conn;
 
                 EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
@@ -273,7 +275,7 @@ int main(int argc, char **argv)
             /* Test that s2n_server_key_share_extension.recv is a no-op
              * if tls1.3 not enabled */
             {
-                struct s2n_stuffer extension_stuffer;
+                struct s2n_stuffer extension_stuffer = { 0 };
                 struct s2n_connection *client_conn;
 
                 EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
@@ -309,7 +311,7 @@ int main(int argc, char **argv)
 
         /* Test error handling parsing broken/trancated p256 key share */
         {
-            struct s2n_stuffer extension_stuffer;
+            struct s2n_stuffer extension_stuffer = { 0 };
             struct s2n_connection *client_conn;
 
             EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
@@ -326,7 +328,7 @@ int main(int argc, char **argv)
 
         /* Test failure for receiving p256 key share for client configured p384 key share */
         {
-            struct s2n_stuffer extension_stuffer;
+            struct s2n_stuffer extension_stuffer = { 0 };
             struct s2n_connection *client_conn;
 
             EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
@@ -355,7 +357,7 @@ int main(int argc, char **argv)
     /* Test Shared Key Generation */
     {
         struct s2n_connection *client_conn, *server_conn;
-        struct s2n_stuffer key_share_extension;
+        struct s2n_stuffer key_share_extension = { 0 };
 
         EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
@@ -471,6 +473,7 @@ int main(int argc, char **argv)
             struct s2n_stuffer *key_share_extension = &server_conn->handshake.io;
 
             EXPECT_SUCCESS(s2n_connection_set_all_protocol_versions(server_conn, S2N_TLS13));
+            EXPECT_OK(s2n_conn_choose_state_machine(server_conn, S2N_TLS13));
             client_conn->security_policy_override = &security_policy_test_tls13_retry;
 
             server_conn->kex_params.server_ecc_evp_params.negotiated_curve = s2n_all_supported_curves_list[0];
@@ -521,6 +524,7 @@ int main(int argc, char **argv)
             .kems = NULL,
             .tls13_kem_group_count = s2n_array_len(test_kem_groups),
             .tls13_kem_groups = test_kem_groups,
+            .tls13_pq_hybrid_draft_revision = 0
         };
 
         const struct s2n_security_policy test_security_policy = {
@@ -536,6 +540,7 @@ int main(int argc, char **argv)
             .kems = NULL,
             .tls13_kem_group_count = S2N_SUPPORTED_KEM_GROUPS_COUNT,
             .tls13_kem_groups = ALL_SUPPORTED_KEM_GROUPS,
+            .tls13_pq_hybrid_draft_revision = 0
         };
 
         const struct s2n_security_policy test_all_supported_kems_security_policy = {
@@ -555,6 +560,7 @@ int main(int argc, char **argv)
             .kems = NULL,
             .tls13_kem_group_count = s2n_array_len(kem_groups_kyber),
             .tls13_kem_groups = kem_groups_kyber,
+            .tls13_pq_hybrid_draft_revision = 0
         };
 
         const struct s2n_security_policy security_policy_kyber = {
@@ -618,6 +624,7 @@ int main(int argc, char **argv)
                         client_conn->kex_params.client_kem_group_params.kem_group = kem_group;
                         client_conn->kex_params.client_kem_group_params.ecc_params.negotiated_curve = kem_group->curve;
                         client_conn->kex_params.client_kem_group_params.kem_params.kem = kem_group->kem;
+                        client_conn->kex_params.client_kem_group_params.kem_params.len_prefixed = true;
                         EXPECT_SUCCESS(s2n_ecc_evp_generate_ephemeral_key(&client_conn->kex_params.client_kem_group_params.ecc_params));
 
                         /* Call the function and assert correctness */
@@ -651,6 +658,7 @@ int main(int argc, char **argv)
                     EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
                     client_conn->security_policy_override = &test_security_policy;
                     client_conn->actual_protocol_version = S2N_TLS13;
+                    EXPECT_OK(s2n_conn_choose_state_machine(client_conn, S2N_TLS13));
                     client_conn->handshake.handshake_type = HELLO_RETRY_REQUEST;
                     client_conn->handshake.message_number = HELLO_RETRY_MSG_NO;
                     client_conn->actual_protocol_version_established = 1;
@@ -723,6 +731,7 @@ int main(int argc, char **argv)
                         client_conn->kex_params.client_kem_group_params.kem_group = kem_group;
                         client_conn->kex_params.client_kem_group_params.ecc_params.negotiated_curve = kem_group->curve;
                         client_conn->kex_params.client_kem_group_params.kem_params.kem = kem_group->kem;
+                        client_conn->kex_params.client_kem_group_params.kem_params.len_prefixed = true;
                         EXPECT_SUCCESS(s2n_ecc_evp_generate_ephemeral_key(&client_conn->kex_params.client_kem_group_params.ecc_params));
                         EXPECT_FAILURE_WITH_ERRNO(s2n_server_key_share_extension.recv(client_conn, &key_share_payload),
                                 S2N_ERR_BAD_KEY_SHARE);
@@ -831,58 +840,71 @@ int main(int argc, char **argv)
 
         /* Test s2n_server_key_share_extension.send sends key share success (PQ) */
         if (s2n_pq_is_enabled()) {
-            for (size_t i = 0; i < S2N_SUPPORTED_KEM_GROUPS_COUNT; i++) {
-                struct s2n_connection *conn = NULL;
-                EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_SERVER));
-                conn->security_policy_override = &test_all_supported_kems_security_policy;
+            for (int len_prefixed = 0; len_prefixed < 2; len_prefixed++) {
+                for (size_t i = 0; i < S2N_SUPPORTED_KEM_GROUPS_COUNT; i++) {
+                    struct s2n_connection *conn = NULL;
+                    EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_SERVER));
+                    conn->security_policy_override = &test_all_supported_kems_security_policy;
 
-                const struct s2n_kem_preferences *kem_pref = NULL;
-                EXPECT_SUCCESS(s2n_connection_get_kem_preferences(conn, &kem_pref));
-                EXPECT_NOT_NULL(kem_pref);
+                    const struct s2n_kem_preferences *kem_pref = NULL;
+                    EXPECT_SUCCESS(s2n_connection_get_kem_preferences(conn, &kem_pref));
+                    EXPECT_NOT_NULL(kem_pref);
 
-                DEFER_CLEANUP(struct s2n_stuffer stuffer = { 0 }, s2n_stuffer_free);
-                EXPECT_SUCCESS(s2n_stuffer_growable_alloc(&stuffer, 4096));
+                    DEFER_CLEANUP(struct s2n_stuffer stuffer = { 0 }, s2n_stuffer_free);
+                    EXPECT_SUCCESS(s2n_stuffer_growable_alloc(&stuffer, 4096));
 
-                /* Set up the server so that it's chosen the correct KEM group and received
-                 * a correspond keyshare from client */
-                conn->kex_params.server_ecc_evp_params.negotiated_curve = NULL;
+                    /* Set up the server so that it's chosen the correct KEM group and received
+                     * a correspond keyshare from client */
+                    conn->kex_params.server_ecc_evp_params.negotiated_curve = NULL;
 
-                struct s2n_kem_group_params *server_params = &conn->kex_params.server_kem_group_params;
-                const struct s2n_kem_group *kem_group = kem_pref->tls13_kem_groups[i];
-                server_params->kem_group = kem_group;
-                server_params->kem_params.kem = kem_group->kem;
-                server_params->ecc_params.negotiated_curve = kem_group->curve;
+                    struct s2n_kem_group_params *server_params = &conn->kex_params.server_kem_group_params;
+                    const struct s2n_kem_group *kem_group = kem_pref->tls13_kem_groups[i];
+                    server_params->kem_group = kem_group;
+                    server_params->kem_params.kem = kem_group->kem;
+                    server_params->ecc_params.negotiated_curve = kem_group->curve;
 
-                struct s2n_kem_group_params *client_params = &conn->kex_params.client_kem_group_params;
-                client_params->kem_group = kem_group;
-                client_params->kem_params.kem = kem_group->kem;
-                client_params->ecc_params.negotiated_curve = kem_group->curve;
+                    struct s2n_kem_group_params *client_params = &conn->kex_params.client_kem_group_params;
+                    client_params->kem_group = kem_group;
+                    client_params->kem_params.kem = kem_group->kem;
+                    client_params->kem_params.len_prefixed = (bool) len_prefixed; /* This would normally be auto-detected when receiving the Client's KeyShare*/
+                    client_params->ecc_params.negotiated_curve = kem_group->curve;
 
-                EXPECT_SUCCESS(s2n_ecc_evp_generate_ephemeral_key(&client_params->ecc_params));
-                EXPECT_SUCCESS(s2n_alloc(&client_params->kem_params.public_key, kem_group->kem->public_key_length));
-                EXPECT_OK(s2n_kem_generate_keypair(&client_params->kem_params));
+                    EXPECT_SUCCESS(s2n_ecc_evp_generate_ephemeral_key(&client_params->ecc_params));
+                    EXPECT_SUCCESS(s2n_alloc(&client_params->kem_params.public_key, kem_group->kem->public_key_length));
+                    EXPECT_OK(s2n_kem_generate_keypair(&client_params->kem_params));
+                    EXPECT_SUCCESS(s2n_server_key_share_extension.send(conn, &stuffer));
+                    uint16_t expected_hybrid_share_size = kem_group->curve->share_size + kem_group->kem->ciphertext_length;
 
-                EXPECT_SUCCESS(s2n_server_key_share_extension.send(conn, &stuffer));
+                    if (client_params->kem_params.len_prefixed) {
+                        expected_hybrid_share_size += (2 * S2N_SIZE_OF_KEY_SHARE_SIZE);
+                    }
 
-                /* IANA ID (2 bytes) + total share size (2 bytes) + server hybrid share */
-                EXPECT_EQUAL(s2n_stuffer_data_available(&stuffer), (2 * sizeof(uint16_t)) + kem_group->server_share_size);
+                    /*  IANA ID (2 bytes) + total share size (2 bytes) + Hybrid Share */
+                    EXPECT_EQUAL(s2n_stuffer_data_available(&stuffer), ((2 * sizeof(uint16_t)) + expected_hybrid_share_size));
 
-                /* Assert we sent a hybrid key share */
-                S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, kem_group->iana_id, uint16);
-                S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, kem_group->server_share_size, uint16);
-                S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, kem_group->curve->share_size, uint16);
-                EXPECT_SUCCESS(s2n_stuffer_skip_read(&stuffer, kem_group->curve->share_size));
-                S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, kem_group->kem->ciphertext_length, uint16);
-                S2N_STUFFER_LENGTH_WRITTEN_EXPECT_EQUAL(&stuffer, kem_group->kem->ciphertext_length);
+                    /* Assert we sent a hybrid key share */
+                    S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, kem_group->iana_id, uint16);
+                    S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, expected_hybrid_share_size, uint16);
 
-                EXPECT_NULL(conn->kex_params.server_ecc_evp_params.negotiated_curve);
-                EXPECT_EQUAL(server_params->kem_group, kem_group);
-                EXPECT_EQUAL(server_params->kem_params.kem, kem_group->kem);
-                EXPECT_EQUAL(server_params->ecc_params.negotiated_curve, kem_group->curve);
-                EXPECT_EQUAL(client_params->kem_params.shared_secret.size, kem_group->kem->shared_secret_key_length);
-                EXPECT_NOT_NULL(client_params->kem_params.shared_secret.data);
+                    if (len_prefixed) {
+                        S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, kem_group->curve->share_size, uint16);
+                    }
+                    EXPECT_SUCCESS(s2n_stuffer_skip_read(&stuffer, kem_group->curve->share_size));
 
-                EXPECT_SUCCESS(s2n_connection_free(conn));
+                    if (len_prefixed) {
+                        S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, kem_group->kem->ciphertext_length, uint16);
+                    }
+                    S2N_STUFFER_LENGTH_WRITTEN_EXPECT_EQUAL(&stuffer, kem_group->kem->ciphertext_length);
+
+                    EXPECT_NULL(conn->kex_params.server_ecc_evp_params.negotiated_curve);
+                    EXPECT_EQUAL(server_params->kem_group, kem_group);
+                    EXPECT_EQUAL(server_params->kem_params.kem, kem_group->kem);
+                    EXPECT_EQUAL(server_params->ecc_params.negotiated_curve, kem_group->curve);
+                    EXPECT_EQUAL(client_params->kem_params.shared_secret.size, kem_group->kem->shared_secret_key_length);
+                    EXPECT_NOT_NULL(client_params->kem_params.shared_secret.data);
+
+                    EXPECT_SUCCESS(s2n_connection_free(conn));
+                }
             }
         }
 
@@ -892,6 +914,7 @@ int main(int argc, char **argv)
             EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_SERVER));
             conn->security_policy_override = &test_all_supported_kems_security_policy;
             conn->actual_protocol_version = S2N_TLS13;
+            EXPECT_OK(s2n_conn_choose_state_machine(conn, S2N_TLS13));
             conn->handshake.handshake_type = HELLO_RETRY_REQUEST;
             conn->handshake.message_number = HELLO_RETRY_MSG_NO;
 
